@@ -30,6 +30,7 @@
 #include <libgnomeui/gnome-about.h>
 #include <gconf/gconf-client.h>
 #include <gdk/gdkkeysyms.h>
+#include <libgnomeui/gnome-help.h>
 
 #include "netstatus-icon.h"
 #include "netstatus-iface.h"
@@ -48,26 +49,33 @@ struct _NetstatusAppletPrivate
   guint           notify_id;
 };
 
-static void     netstatus_applet_instance_init             (NetstatusApplet      *applet,
-							    NetstatusAppletClass *klass);
-static void     netstatus_applet_class_init                (NetstatusAppletClass *klass);
-static void     netstatus_applet_finalize                  (GObject              *object);
-static gboolean netstatus_applet_key_press_event           (GtkWidget            *widget,
-							    GdkEventKey          *event);
-static void     netstatus_applet_orientation_changed       (NetstatusApplet      *applet,
-							    PanelAppletOrient     orient);
-static void     netstatus_applet_display_about_dialog      (BonoboUIComponent    *uic,
-							    NetstatusApplet      *applet);
-static void     netstatus_applet_display_properties_dialog (BonoboUIComponent    *uic,
-							    NetstatusApplet      *applet);
-static void     netstatus_applet_display_status_dialog     (NetstatusApplet      *applet);
-static void     netstatus_applet_iface_name_changed        (NetstatusApplet      *applet);
+static void     netstatus_applet_instance_init             (NetstatusApplet           *applet,
+							    NetstatusAppletClass      *klass);
+static void     netstatus_applet_class_init                (NetstatusAppletClass      *klass);
+static void     netstatus_applet_finalize                  (GObject                   *object);
+static gboolean netstatus_applet_key_press_event           (GtkWidget                 *widget,
+							    GdkEventKey               *event);
+static void     netstatus_applet_orientation_changed       (NetstatusApplet           *applet,
+							    PanelAppletOrient          orient);
+static void     netstatus_applet_background_changed        (NetstatusApplet           *applet,
+							    PanelAppletBackgroundType  type,
+							    GdkColor                  *colour,
+							    GdkPixmap                 *pixmap);
+static void     netstatus_applet_display_help              (BonoboUIComponent         *uic,
+							    NetstatusApplet           *applet);
+static void     netstatus_applet_display_about_dialog      (BonoboUIComponent         *uic,
+							    NetstatusApplet           *applet);
+static void     netstatus_applet_display_properties_dialog (BonoboUIComponent         *uic,
+							    NetstatusApplet           *applet);
+static void     netstatus_applet_display_status_dialog     (NetstatusApplet           *applet);
+static void     netstatus_applet_iface_name_changed        (NetstatusApplet           *applet);
 
 static GObjectClass *parent_class;
 
 static const BonoboUIVerb netstatus_menu_verbs [] =
   {
     BONOBO_UI_UNSAFE_VERB ("NetstatusProperties", netstatus_applet_display_properties_dialog),
+    BONOBO_UI_UNSAFE_VERB ("NetstatusHelp", netstatus_applet_display_help),
     BONOBO_UI_UNSAFE_VERB ("NetstatusAbout", netstatus_applet_display_about_dialog),
     
     BONOBO_UI_VERB_END
@@ -106,6 +114,8 @@ static void
 netstatus_applet_instance_init (NetstatusApplet      *applet,
 				NetstatusAppletClass *klass)
 {
+  AtkObject *obj;
+
   applet->priv = g_new0 (NetstatusAppletPrivate, 1);
 
   applet->priv->client = gconf_client_get_default ();
@@ -121,6 +131,9 @@ netstatus_applet_instance_init (NetstatusApplet      *applet,
 			    G_CALLBACK (netstatus_applet_display_status_dialog),
 			    applet);
 
+  obj = gtk_widget_get_accessible (GTK_WIDGET (applet));
+  atk_object_set_name (obj, _("Network Monitor"));
+
   gtk_container_add (GTK_CONTAINER (applet), applet->priv->icon);
   gtk_widget_show (applet->priv->icon);
 
@@ -128,6 +141,8 @@ netstatus_applet_instance_init (NetstatusApplet      *applet,
 
   g_signal_connect (applet, "change-orient",
 		    G_CALLBACK (netstatus_applet_orientation_changed), NULL);
+  g_signal_connect (applet, "change_background",
+		    G_CALLBACK (netstatus_applet_background_changed), NULL);
 }
 
 static void
@@ -191,8 +206,10 @@ netstatus_applet_key_press_event (GtkWidget   *widget,
       netstatus_icon_invoke (NETSTATUS_ICON (applet->priv->icon));
       return TRUE;
     default:
-      return FALSE;
+      break;
     }
+
+  return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
 }
 
 static void
@@ -226,6 +243,32 @@ netstatus_applet_display_status_dialog (NetstatusApplet *applet)
   gtk_widget_show (GTK_WIDGET (applet->priv->status_dialog));
 }
 
+static void
+netstatus_applet_display_help (BonoboUIComponent *uic,
+			       NetstatusApplet   *applet)
+{
+  GError *error = NULL;
+  
+  gnome_help_display_on_screen ("gnome-netstatus", NULL,
+				gtk_widget_get_screen (GTK_WIDGET (applet)),
+				&error);
+  if (error)
+    {
+      GtkWidget *message_dialog;
+      message_dialog = gtk_message_dialog_new (NULL,
+					       0,
+					       GTK_MESSAGE_ERROR,
+					       GTK_BUTTONS_OK,
+					       _("There was an error displaying help: %s"),
+					       error->message);
+      g_error_free (error);
+      gtk_window_set_screen (GTK_WINDOW (message_dialog),
+			     gtk_widget_get_screen (GTK_WIDGET (applet)));
+      gtk_dialog_run (GTK_DIALOG (message_dialog));
+      gtk_widget_destroy (message_dialog);
+    }
+}
+
 static void
 netstatus_applet_display_about_dialog (BonoboUIComponent *uic,
 				       NetstatusApplet   *applet)
@@ -282,6 +325,55 @@ netstatus_applet_display_about_dialog (BonoboUIComponent *uic,
   gtk_widget_show (applet->priv->about_dialog);
 }
 
+static void
+netstatus_applet_background_changed (NetstatusApplet		*applet,
+				     PanelAppletBackgroundType	 type,
+				     GdkColor			*colour,
+				     GdkPixmap			*pixmap)
+{
+	GtkRcStyle *rc_style;
+
+	gtk_widget_set_style (GTK_WIDGET (applet), NULL);
+	gtk_widget_set_style (GTK_WIDGET (applet->priv->icon), NULL);
+
+	rc_style = gtk_rc_style_new ();
+	gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
+	gtk_widget_modify_style (GTK_WIDGET (applet->priv->icon), rc_style);
+	g_object_unref (rc_style);
+
+	switch (type) {
+	case PANEL_NO_BACKGROUND:
+		break;
+
+	case PANEL_COLOR_BACKGROUND:
+		gtk_widget_modify_bg (GTK_WIDGET (applet),
+				      GTK_STATE_NORMAL,
+				      colour);
+		gtk_widget_modify_bg (GTK_WIDGET (applet->priv->icon),
+				      GTK_STATE_NORMAL,
+				      colour);
+		break;
+
+	case PANEL_PIXMAP_BACKGROUND: {
+		GtkStyle *style;
+
+		style = gtk_style_copy (GTK_WIDGET (applet)->style);
+
+		if (style->bg_pixmap [GTK_STATE_NORMAL])
+			g_object_unref (style->bg_pixmap [GTK_STATE_NORMAL]);
+		style->bg_pixmap [GTK_STATE_NORMAL] = g_object_ref (pixmap);
+
+		gtk_widget_set_style (GTK_WIDGET (applet), style);
+		gtk_widget_set_style (GTK_WIDGET (applet->priv->icon), style);
+		break;
+	}
+
+	default:
+		g_assert_not_reached();
+		break;
+	}
+}
+
 static void
 netstatus_applet_orientation_changed (NetstatusApplet   *applet,
 				      PanelAppletOrient  orient)
@@ -335,7 +427,8 @@ netstatus_applet_create (NetstatusApplet *applet,
   char *iface_name;
   char *key;
 
-  if (strcmp (iid, "OAFIID:GNOME_NetstatusApplet") != 0)
+  if (strcmp (iid, "OAFIID:GNOME_NetstatusApplet") != 0 &&
+      strcmp (iid, "OAFIID:GNOME_Panel_WirelessApplet") != 0)
     return FALSE;
 
   panel_applet_set_flags (PANEL_APPLET (applet), PANEL_APPLET_EXPAND_MINOR);
